"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Container_types, _Container_instances, _Container_singletones, _Container_builderFactory;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
const CircularDependencyError_1 = require("./CircularDependencyError");
const TInstanceType_1 = require("./TInstanceType");
function isClass(func) {
    return typeof func === 'function'
        && Function.prototype.toString.call(func).startsWith('class');
}
function extendContainer(container, additionalParameters) {
    if (!container)
        throw new TypeError('container argument required');
    if (typeof additionalParameters !== 'object' || !additionalParameters)
        throw new TypeError('additionalArguments argument must be an Object');
    const paramDescriptors = Object.getOwnPropertyDescriptors(additionalParameters);
    return Object.create(container, paramDescriptors);
}
class Container {
    constructor({ types, singletones, builderFactory }) {
        _Container_types.set(this, void 0);
        _Container_instances.set(this, {});
        _Container_singletones.set(this, void 0);
        _Container_builderFactory.set(this, void 0);
        /** Type aliases, stacked on each type instantiation */
        this._dependencyStack = [];
        __classPrivateFieldSet(this, _Container_types, types, "f");
        __classPrivateFieldSet(this, _Container_singletones, singletones, "f");
        __classPrivateFieldSet(this, _Container_builderFactory, builderFactory, "f");
        for (const { aliases } of __classPrivateFieldGet(this, _Container_types, "f")) {
            for (const alias of aliases) {
                Object.defineProperty(this, alias, {
                    get: () => this.get(alias),
                    configurable: true,
                    enumerable: true
                });
            }
        }
        // bind container methods to container
        // to allow their usage in dependent constructors and factories
        const methodsToBind = [this.get, this.getAll, this.createInstance];
        for (const method of methodsToBind) {
            Object.defineProperty(this, method.name, {
                value: method.bind(this),
                configurable: true,
                enumerable: false,
                writable: false
            });
        }
        for (const { id, factory } of __classPrivateFieldGet(this, _Container_types, "f").filter(t => !t.aliases.length))
            __classPrivateFieldGet(this, _Container_instances, "f")[id] = factory(this);
    }
    /**
     * Get instance by alias
     */
    get(alias) {
        var _a;
        if (!alias)
            throw new TypeError('alias argument required');
        const types = __classPrivateFieldGet(this, _Container_types, "f").filter(t => t.id === alias || typeof alias === 'string' && t.aliases.includes(alias));
        if (!types.length)
            throw new Error(`alias "${alias}" is not registered`);
        const { id, instanceType, factory } = types[types.length - 1];
        if (__classPrivateFieldGet(this, _Container_singletones, "f")[id])
            return __classPrivateFieldGet(this, _Container_singletones, "f")[id];
        if (__classPrivateFieldGet(this, _Container_instances, "f")[id])
            return __classPrivateFieldGet(this, _Container_instances, "f")[id];
        let instance;
        if (typeof alias === 'string') {
            if (this._dependencyStack.includes(alias))
                throw new CircularDependencyError_1.default([...this._dependencyStack, alias]);
            this._dependencyStack.push(alias);
            try {
                instance = factory(this);
                if (alias !== 'logger')
                    (_a = this.logger) === null || _a === void 0 ? void 0 : _a.log('silly', `${this._dependencyStack.join('.')} instance created`);
            }
            finally {
                this._dependencyStack.pop();
            }
        }
        else {
            instance = factory(this);
        }
        if (instanceType === TInstanceType_1.INSTANCE_SINGLE)
            __classPrivateFieldGet(this, _Container_singletones, "f")[id] = instance;
        else if (instanceType === TInstanceType_1.INSTANCE_PER_CONTAINER)
            __classPrivateFieldGet(this, _Container_instances, "f")[id] = instance;
        return instance;
    }
    /**
     * Get all instances by alias
     */
    getAll(alias) {
        if (!alias)
            throw new TypeError('alias argument required');
        const types = __classPrivateFieldGet(this, _Container_types, "f").filter(t => t.aliases.includes(alias));
        if (!types.length)
            throw new Error(`alias "${alias}" is not registered`);
        return types.map(({ id }) => this.get(id));
    }
    /**
     * Create instance of a given type
     */
    createInstance(Type, additionalParams) {
        const arg = additionalParams ?
            extendContainer(this, additionalParams) :
            this;
        if (isClass(Type))
            return new Type(arg);
        return Type(arg);
    }
    /**
     * Create an instance of ContainerBuilder for container extension
     */
    builder() {
        return __classPrivateFieldGet(this, _Container_builderFactory, "f").call(this, {
            singletones: __classPrivateFieldGet(this, _Container_singletones, "f")
        });
    }
}
exports.Container = Container;
_Container_types = new WeakMap(), _Container_instances = new WeakMap(), _Container_singletones = new WeakMap(), _Container_builderFactory = new WeakMap();
//# sourceMappingURL=Container.js.map