"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeConfig = void 0;
const Container_1 = require("./Container");
const TInstanceType_1 = require("./TInstanceType");
class TypeConfig {
    /**
     * Creates an instance of TypeConfig<T>
     */
    constructor(Type) {
        /** List of type aliases */
        this.aliases = [];
        /** How to instantiate the type */
        this.instanceType = TInstanceType_1.INSTANCE_PER_CONTAINER;
        if (typeof Type !== 'function')
            throw new TypeError('Type argument must be a Function');
        if (Type.length > 1)
            throw new TypeError('Type cannot have more than 1 argument');
        this.id = Symbol(Type.name);
        this.factory = container => container.createInstance(Type);
    }
    /**
     * Instruct to expose object instance on container instance with a given `alias`.
     * The alias will be used to inject object instance as dependency to other types.
     */
    as(alias) {
        if (typeof alias !== 'string' || !alias.length)
            throw new TypeError('Alias argument must be a non-empty String');
        if (this.aliases.includes(alias))
            throw new TypeError(`Alias "${alias}" is already registered for the type`);
        const forbiddenAliases = [
            Container_1.Container.prototype.get.name,
            Container_1.Container.prototype.getAll.name,
            Container_1.Container.prototype.createInstance.name
        ];
        if (forbiddenAliases.includes(alias))
            throw new TypeError(`Alias "${alias}" conflicts with container method`);
        this.aliases.push(alias);
        return this;
    }
    /**
     * Instruct to create object instances once per containers tree
     * (current container and derived containers)
     */
    asSingleInstance() {
        this.instanceType = TInstanceType_1.INSTANCE_SINGLE;
        return this;
    }
    /**
     * Create instance per each dependency
     */
    asInstancePerDependency() {
        this.instanceType = TInstanceType_1.INSTANCE_PER_DEPENDENCY;
        return this;
    }
    /**
     * Create instance per container (default behavior)
     */
    asInstancePerContainer() {
        this.instanceType = TInstanceType_1.INSTANCE_PER_CONTAINER;
        return this;
    }
}
exports.TypeConfig = TypeConfig;
//# sourceMappingURL=TypeConfig.js.map